//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskItems.h
//! @brief     Defines classes MaskItem, RectangleItem, ..., MaskAllItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MASK_MASKITEMS_H
#define BORNAGAIN_GUI_MODEL_MASK_MASKITEMS_H

#include "Base/Types/OwningVector.h"
#include "GUI/Model/Descriptor/SelectionProperty.h"
#include "GUI/Model/Mask/MaskObject.h"

class IShape2D;

//! A base class for mask items that describe one specific shape.

class MaskItem : public MaskObject {
public:
    ~MaskItem() override;

    virtual std::unique_ptr<IShape2D> createShape() const = 0;

    QString maskName() const { return m_maskName; }
    void setMaskName(const QString& name);

    bool maskValue() const { return m_maskValue; }
    void setMaskValue(bool mask_value);

    bool isVisible() const { return m_isVisible; }
    void setIsVisible(bool visible);

    bool wasVisible() const { return m_wasVisible; }
    void setWasVisible(bool v) { m_wasVisible = v; }

    virtual void writeTo(QXmlStreamWriter* w) const;
    virtual void readFrom(QXmlStreamReader* r);

protected:
    explicit MaskItem();
    QString m_maskName = "nameless by default";
    bool m_maskValue = true;
    bool m_isVisible = true;
    bool m_wasVisible = m_isVisible; // do not serialize
};

class RectangleItem : public MaskItem {
public:
    explicit RectangleItem();
    std::unique_ptr<IShape2D> createShape() const override;

    double xLow() const { return m_xLow; }
    void setXLow(double val);
    double yLow() const { return m_yLow; }
    void setYLow(double val);
    double xUp() const { return m_xUp; }
    void setXUp(double val);
    double yUp() const { return m_yUp; }
    void setYUp(double val);

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

protected:
    explicit RectangleItem(const QString& modelType);

private:
    double m_xLow;
    double m_yLow;
    double m_xUp;
    double m_yUp;
};

class RegionOfInterestItem : public RectangleItem {
public:
    RegionOfInterestItem();
    std::unique_ptr<IShape2D> createShape() const override;
};

class PolygonPointItem : public MaskObject {
public:
    static constexpr auto M_TYPE{"PolygonPoint"};

    PolygonPointItem();

    double posX() const { return m_posX; }
    void setPosX(double val);
    double posY() const { return m_posY; }
    void setPosY(double val);

    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r);

private:
    double m_posX = 0;
    double m_posY = 0;
};

class PolygonItem : public MaskItem {
public:
    PolygonItem();
    std::unique_ptr<IShape2D> createShape() const override;

    bool isClosed() const { return m_isClosed; }
    void setIsClosed(bool closed);

    QVector<PolygonPointItem*> points() const;
    void addPoint(double x, double y);

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

private:
    bool m_isClosed = false;
    OwningVector<PolygonPointItem> m_points;
};

class VerticalLineItem : public MaskItem {
public:
    VerticalLineItem();
    std::unique_ptr<IShape2D> createShape() const override;

    double posX() const { return m_posX; }
    void setPosX(double val);

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

private:
    double m_posX = 0;
};

class HorizontalLineItem : public MaskItem {
public:
    HorizontalLineItem();
    std::unique_ptr<IShape2D> createShape() const override;

    double posY() const { return m_posY; }
    void setPosY(double pos_y);

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

private:
    double m_posY = 0;
};

class EllipseItem : public MaskItem {
public:
    EllipseItem();
    std::unique_ptr<IShape2D> createShape() const override;

    double xCenter() const { return m_xCenter; }
    void setXCenter(double val);

    double yCenter() const { return m_yCenter; }
    void setYCenter(double val);

    double xRadius() const { return m_xRadius; }
    void setXRadius(double val);

    double yRadius() const { return m_yRadius; }
    void setYRadius(double val);

    double angle() const { return m_angle; }
    void setAngle(double val);

    void writeTo(QXmlStreamWriter* w) const override;
    void readFrom(QXmlStreamReader* r) override;

private:
    double m_xCenter = 0;
    double m_yCenter = 0;
    double m_xRadius = 0;
    double m_yRadius = 0;
    double m_angle = 0;
};

class MaskAllItem : public MaskItem {
public:
    MaskAllItem();
    std::unique_ptr<IShape2D> createShape() const override;
};

#endif // BORNAGAIN_GUI_MODEL_MASK_MASKITEMS_H
